#include "stm32f10x.h"
#include "max7219.h"
#include <stdio.h>
//----------------------------------------------------------
#define SYSCLOCK 72000000U
//----------------------------------------------------------
#define CS2_RESET() SET_BIT(GPIOB->ODR,GPIO_ODR_ODR12)
#define CS2_SET() CLEAR_BIT(GPIOB->ODR,GPIO_ODR_ODR12)
#define SPI2_ENABLE() SET_BIT(SPI2->CR1, SPI_CR1_SPE);
//----------------------------------------------------------
__IO uint32_t tmpreg;
__IO uint32_t SysTick_CNT = 0;
char str1[30];
//----------------------------------------------------------
__forceinline void delay(__IO uint32_t tck)
{
  while(tck)
  {
    tck--;
  }  
}
//----------------------------------------------------------
void delay_ms(uint32_t ms)
{
  MODIFY_REG(SysTick->VAL,SysTick_VAL_CURRENT_Msk,SYSCLOCK / 1000 - 1);
  SysTick_CNT = ms;
  while(SysTick_CNT) {}
}
//-----------------------------------------------------------
void RCC_DeInit(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSION);
  while(READ_BIT(RCC->CR, RCC_CR_HSIRDY == RESET)) {}
  MODIFY_REG(RCC->CR, RCC_CR_HSITRIM, 0x80U);
  CLEAR_REG(RCC->CFGR);
  while (READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_PLLON);
  while (READ_BIT(RCC->CR, RCC_CR_PLLRDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEON | RCC_CR_CSSON);
  while (READ_BIT(RCC->CR, RCC_CR_HSERDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEBYP);
  //Reset all CSR flags
  SET_BIT(RCC->CSR, RCC_CSR_RMVF);
}
//----------------------------------------------------------
void SetSysClockTo72(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSEON);
  while(READ_BIT(RCC->CR, RCC_CR_HSERDY == RESET)) {}
  //Enable the Prefetch Buffer
  CLEAR_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  SET_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  MODIFY_REG(FLASH->ACR, FLASH_ACR_LATENCY, FLASH_ACR_LATENCY_2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL,
             RCC_CFGR_PLLSRC | RCC_CFGR_PLLMULL9);
  SET_BIT(RCC->CR, RCC_CR_PLLON);
  while(READ_BIT(RCC->CR, RCC_CR_PLLRDY) != (RCC_CR_PLLRDY)) {}
  MODIFY_REG(RCC->CFGR, RCC_CFGR_SW, RCC_CFGR_SW_PLL);
  while(READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) {}
}
//----------------------------------------------------------
void SysTick_Init(void)
{
  MODIFY_REG(SysTick->LOAD,SysTick_LOAD_RELOAD_Msk,SYSCLOCK / 1000 - 1);
  CLEAR_BIT(SysTick->VAL, SysTick_VAL_CURRENT_Msk);
  SET_BIT(SysTick->CTRL, SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk | SysTick_CTRL_TICKINT_Msk);
}
//----------------------------------------------------------
static void GPIO_Init(void)
{
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPAEN | RCC_APB2ENR_IOPBEN);
  MODIFY_REG(GPIOA->CRL,  GPIO_CRL_CNF4_1 | GPIO_CRL_CNF4_0 | GPIO_CRL_MODE4_1, GPIO_CRL_MODE4_0);
  MODIFY_REG(GPIOB->CRH,  GPIO_CRH_CNF12_1 | GPIO_CRH_CNF12_0 | GPIO_CRH_MODE12_1, GPIO_CRH_MODE12_0);
}
//----------------------------------------------------------
static void SPI2_Init(void)
{
  //SPI2 GPIO
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPBEN);
  MODIFY_REG(GPIOB->CRH, GPIO_CRH_CNF13_0 | GPIO_CRH_CNF14_1 | GPIO_CRH_CNF15_0 | GPIO_CRH_MODE14, 
                          GPIO_CRH_CNF13_1 | GPIO_CRH_CNF14_0 | GPIO_CRH_CNF15_1 | GPIO_CRH_MODE13 | GPIO_CRH_MODE15);
  //RCC peripheral clock enabling
  SET_BIT(RCC->APB1ENR, RCC_APB1ENR_SPI2EN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB1ENR, RCC_APB1ENR_SPI2EN);
  //8 bit 0:0
//  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_DFF | SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA , \
//                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_BR_0 | SPI_CR1_MSTR);
  //16 bit 0:0
//  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA , \
//                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_DFF | SPI_CR1_BR_0 | SPI_CR1_MSTR);
  //16 bit 1:1
  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | SPI_CR1_BR_2, SPI_CR1_BR_0 | \
                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_DFF | \
                        SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA);
  CLEAR_BIT(SPI2->CR2, 0x00FF);
}
//----------------------------------------------------------
void USART_TX (uint8_t* dt, uint16_t sz)
{
  uint16_t ind = 0;
  while (ind<sz)
  {
    while (READ_BIT(USART1->SR, USART_SR_TXE) != (USART_SR_TXE)) {}
    USART1->DR = (uint16_t)dt[ind];
    ind++;
  }
}
//----------------------------------------------------------
static void USART1_Init(void)
{
  //USART Clock
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);
  //Set GPIO_TX_RX
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPAEN);
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPAEN);
  MODIFY_REG(GPIOA->CRH, GPIO_CRH_MODE10 | GPIO_CRH_CNF10_1 | GPIO_CRH_CNF9_0,
                          GPIO_CRH_CNF10_0 | GPIO_CRH_MODE9 | GPIO_CRH_CNF9_1);
  ////USART needs to be in disabled state, in order to be able to configure some bits in CRx registers
  if(READ_BIT(USART1->CR1, USART_CR1_UE) != (USART_CR1_UE))
  {
    MODIFY_REG(USART1->CR1,
           USART_CR1_M | USART_CR1_PCE | USART_CR1_PS ,USART_CR1_TE |USART_CR1_RE);
  }
  //Async Mode
  CLEAR_BIT(USART1->CR2, (USART_CR2_LINEN | USART_CR2_CLKEN));
  CLEAR_BIT(USART1->CR3, (USART_CR3_SCEN | USART_CR3_IREN | USART_CR3_HDSEL));
  //115200
  WRITE_REG(USART1->BRR, ((0x27)<<4)|0x01);
  //Enable
  SET_BIT(USART1->CR1, USART_CR1_UE);
}
//----------------------------------------------------------
int main(void)
{
  uint16_t i=0;
  RCC_DeInit();
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //NOJTAG: JTAG-DP Disabled and SW-DP Enabled 
  CLEAR_BIT(AFIO->MAPR,AFIO_MAPR_SWJ_CFG);
  SET_BIT(AFIO->MAPR, AFIO_MAPR_SWJ_CFG_JTAGDISABLE);
  SetSysClockTo72();
  SysTick_Init();
  GPIO_Init();
  SPI2_Init();
  USART1_Init();
  delay_ms(100);
  CS2_RESET();
  SPI2_ENABLE();
  Init_7219();
  Clear_7219();
  Number_7219(87654321);
  delay_ms(2000);
  Clear_7219();
  while(1)
	{
    if(i>1023) i=0;
    sprintf(str1,"String %04d\r\n",i);
    USART_TX((uint8_t*)str1,13);
    NumberR_7219(i);
    delay_ms(100);
    i++;
  }
}
//----------------------------------------------------------
void SysTick_Handler(void)
{
  if(SysTick_CNT > 0)  SysTick_CNT--;
}
//----------------------------------------------------------
